@@ -2,7 +2,7 @@ use {
     crate::{errors::ErrorCode, state::*},
     anchor_lang::prelude::*,
     anchor_spl::token::{self, Token, TokenAccount, Transfer},
-    cardinal_token_manager::{state::{TokenManager, TokenManagerState}, utils::assert_payment_token_account},
+    cardinal_token_manager::{state::{TokenManager, TokenManagerState, PROVIDER_FEE, RECIPIENT_FEE, FEE_SCALE, assert_payment_manager}, utils::assert_payment_token_account},
 };
   
   #[derive(Accounts)]
@@ -15,7 +15,9 @@ use {
   
     #[account(mut, constraint = payment_token_account.mint == use_invalidator.extension_payment_mint.unwrap() @ ErrorCode::InvalidPaymentTokenAccount)]
     payment_token_account: Box<Account<'info, TokenAccount>>,
-  
+    #[account(mut, constraint = payment_manager_token_account.mint == use_invalidator.extension_payment_mint.unwrap() && assert_payment_manager(&payment_manager_token_account.owner) @ ErrorCode::InvalidPaymentManagerTokenAccount)]
+    payment_manager_token_account: Box<Account<'info, TokenAccount>>,
+
     #[account(mut)]
     payer: Signer<'info>,
     #[account(mut, constraint =
@@ -51,6 +53,17 @@ use {
       return Err(error!(ErrorCode::MaxUsagesReached));
     }
   
+    let provider_fee = use_invalidator.extension_payment_amount.unwrap() * (PROVIDER_FEE / FEE_SCALE);
+    let recipient_fee = use_invalidator.extension_payment_amount.unwrap() * (RECIPIENT_FEE / FEE_SCALE);
+    let cpi_accounts = Transfer {
+        from: ctx.accounts.payer_token_account.to_account_info(),
+        to: ctx.accounts.payment_manager_token_account.to_account_info(),
+        authority: ctx.accounts.payer.to_account_info(),
+    };
+    let cpi_program = ctx.accounts.token_program.to_account_info();
+    let cpi_context = CpiContext::new(cpi_program, cpi_accounts);
+    token::transfer(cpi_context, provider_fee + recipient_fee)?;
+  
     let cpi_accounts = Transfer {
       from: ctx.accounts.payer_token_account.to_account_info(),
       to: ctx.accounts.payment_token_account.to_account_info(),
@@ -59,7 +72,7 @@ use {
   
     let cpi_program = ctx.accounts.token_program.to_account_info();
     let cpi_context = CpiContext::new(cpi_program, cpi_accounts);
-    token::transfer(cpi_context, payment_amount)?;
+    token::transfer(cpi_context, payment_amount - recipient_fee)?;
   
     use_invalidator.total_usages = new_total_usages;
     return Ok(());
